@@ -20,9 +20,10 @@ from suite_py.lib.handler.youtrack_handler import (
 
 class Release:
     # pylint: disable=too-many-instance-attributes
-    def __init__(self, project, deploy, config, tokens):
+    def __init__(self, action, project, config, tokens, flags=None):
+        self._action = action
         self._project = project
-        self._deploy = deploy
+        self._flags = flags
         self._config = config
         self._tokens = tokens
         self._youtrack = YoutrackHandler(config, tokens)
@@ -35,21 +36,71 @@ class Release:
 
     def run(self):
         self._stop_if_prod_locked()
-
         self._git.fetch()
 
+        if self._action == "create":
+            self._create()
+        elif self._action == "deploy":
+            self._deploy()
+        else:
+            self._rollback()
+
+    def _parse_available_countries(self):
+        pipelines = self._drone.parse_yaml()
+
+        if pipelines is None:
+            logger.error("The file .drone.yml was not found. Unable to continue.")
+            sys.exit(1)
+
+        countries = []
+        REGEX = re.compile(r"deploy-([a-z]+)-.*")
+        for pipeline in pipelines:
+            if "name" in pipeline:
+                c = REGEX.findall(pipeline["name"])
+                if len(c) > 0 and c[0] is not None and c[0] not in countries:
+                    countries.append(c[0])
+
+        if len(countries) == 0:
+            logger.error(
+                "Can't determine which countries can be deployed. Try to run `suite-py deploy` instead."
+            )
+            sys.exit(1)
+
+        return countries
+
+    def _get_latest(self):
         tags = self._repo.get_tags()
         tag = git.get_last_tag_number(tags)
         latest_release = self._get_release(tag)
 
-        current_version = latest_release.tag_name if latest_release else tag
+        return latest_release.tag_name if latest_release else tag
 
-        if current_version:
-            logger.info(f"The current release is {current_version}")
-            versions = _bump_versions(current_version)
-            commits = self._github.get_commits_since_release(
-                self._repo, current_version
+    def _stop_if_prod_locked(self):
+        request = self._captainhook.status(self._project, "production")
+        if request.status_code != 200:
+            logger.error("Unable to determine lock status on master.")
+            sys.exit(-1)
+
+        request_object = request.json()
+        if request_object["locked"]:
+            logger.error(
+                f"The project is locked in production by {request_object['by']}. Unable to continue."
             )
+            sys.exit(-1)
+
+    def _get_release(self, tag):
+        with Halo(text="Loading...", spinner="dots", color="magenta"):
+            latest_release = self._github.get_latest_release_if_exists(self._repo)
+            if latest_release and latest_release.title == tag:
+                return latest_release
+        return None
+
+    def _create(self):
+        latest = self._get_latest()
+        if latest:
+            logger.info(f"The current release is {latest}")
+            versions = _bump_versions(latest)
+            commits = self._github.get_commits_since_release(self._repo, latest)
 
             _check_migrations_deploy(commits)
 
@@ -83,28 +134,70 @@ class Release:
 
         self._create_release(new_version, message)
 
-        if self._deploy:
-            self._start_deploy(new_version)
+        if "deploy" in self._flags and self._flags["deploy"]:
+            self._deploy(new_version)
 
-    def _stop_if_prod_locked(self):
-        request = self._captainhook.status(self._project, "production")
-        if request.status_code != 200:
-            logger.error("Unable to determine lock status on master.")
-            sys.exit(-1)
+    def _deploy(self, version=""):
+        countries = prompt_utils.ask_multiple_choices(
+            "Which countries do you want to deploy to?",
+            self._countries,
+        )
 
-        request_object = request.json()
-        if request_object["locked"]:
-            logger.error(
-                f"The project is locked in production by {request_object['by']}. Unable to continue."
+        if not version:
+            current = self._get_current_versions(countries)
+            releases = self._get_releases_since(max(rel["version"] for rel in current))
+            if not releases:
+                logger.error(f"No new release found for project {self._project}!")
+                sys.exit(-1)
+            version = prompt_utils.ask_choices(
+                "Which version do you want to deploy?",
+                [{"name": f"{r.tag_name}", "value": r.tag_name} for r in releases],
             )
-            sys.exit(-1)
 
-    def _get_release(self, tag):
-        with Halo(text="Loading...", spinner="dots", color="magenta"):
-            latest_release = self._github.get_latest_release_if_exists(self._repo)
-            if latest_release and latest_release.title == tag:
-                return latest_release
-        return None
+        if not prompt_utils.ask_confirm(
+            f"You're about to deploy {self._project} release {version} in {countries}, do you confirm?",
+            default=False,
+        ):
+            sys.exit(0)
+
+        self._start_deploy(version, countries)
+        commits = self._github.get_commits_since_release(self._repo, version)
+        self._manage_youtrack_card(commits, version, countries)
+
+    def _rollback(self):
+        countries = prompt_utils.ask_multiple_choices(
+            "Which countries do you want to rollback?",
+            self._countries,
+        )
+
+        current = self._get_current_versions(countries)
+        releases = self._get_releases_before(min(rel["version"] for rel in current))
+        if not releases:
+            logger.error(f"No previous releases found for project {self._project}!")
+            sys.exit(-1)
+        version = prompt_utils.ask_choices(
+            "Which version do you want to roll back to?",
+            [{"name": f"{r.tag_name}", "value": r.tag_name} for r in releases],
+        )
+
+        if not prompt_utils.ask_confirm(
+            f"You're about to roll back {self._project} to version {version} in {countries}, do you confirm?",
+            default=False,
+        ):
+            sys.exit(0)
+
+        self._start_deploy(version, countries, rollback=True)
+
+    def _get_current_versions(self, countries):
+        current = self._drone.get_latest_production_deploy(countries)
+        if current:
+            logger.info(f"{self._project} current version:")
+            for c in current:
+                logger.info(f"country {c['country']}: {c['version']}")
+        else:
+            logger.warning(f"No current deploy found for {self._project}")
+
+        return current
 
     def _create_release(self, new_version, message):
         new_release = self._repo.create_git_release(
@@ -118,50 +211,31 @@ class Release:
                 drone_url = self._drone.get_build_url(build_number)
                 logger.info(f"You can follow the build status here: {drone_url}")
 
-    def _start_deploy(self, version):
-        current = self._drone.get_latest_production_deploy()
-        if current:
-            logger.info(f"{self._project} current version:")
-            for c in current:
-                logger.info(f"country {c['country']}: {c['version']}")
-        else:
-            logger.warning(f"No current deploy found for {self._project}")
-
-        countries = prompt_utils.ask_multiple_choices(
-            "Which countries do you want to deploy to?",
-            self._countries,
-        )
+    def _start_deploy(self, version, countries, rollback=False):
         for country in countries:
-            if not version:
-                releases = self._get_latest_releases_since(
-                    [rel["version"] for rel in current if rel["country"] == country]
+            if rollback:
+                promotion = self._drone.promote_production(
+                    version,
+                    f"deploy-{country}-production",
+                    f"DRONE_TAG={version}&ROLLBACK=true",
+                )
+            else:
+                promotion = self._drone.promote_production(
+                    version,
+                    f"deploy-{country}-production",
+                    f"DRONE_TAG={version}",
                 )
 
-                version = prompt_utils.ask_choices(
-                    "Which version do you want to deploy?",
-                    [{"name": f"{r.tag_name}", "value": r.tag_name} for r in releases],
-                )
-
-            promotion = self._drone.promote_production(
-                version,
-                f"deploy-{country}-production",
-                f"DRONE_TAG={version}",
-            )
             if "number" not in promotion:
-                logger.warning(
-                    f"Unable to promote production build. Response: {promotion}"
-                )
+                logger.warning(f"Unable to promote drone build. Response: {promotion}")
                 return
 
-            logger.info("Deploy started successfully!")
+            logger.info("Drone build started successfully!")
             logger.info(
-                f"You can follow the deploy status at {self._drone.get_build_url(promotion['number'])}"
+                f"You can follow the build status here: {self._drone.get_build_url(promotion['number'])}"
             )
 
-            commits = self._github.get_commits_since_release(self._repo, version)
-            self._manage_youtrack_card(commits, version, country)
-
-    def _get_latest_releases_since(self, threshold):
+    def _get_releases_since(self, threshold):
         releases = self._repo.get_releases()
 
         if threshold:
@@ -169,9 +243,19 @@ class Release:
                 rel for rel in releases if semver.compare(threshold, rel.tag_name) == -1
             ]
 
-        return releases
+        return releases.sort(reverse=True)[0:4]
 
-    def _manage_youtrack_card(self, commits, new_version, country):
+    def _get_releases_before(self, threshold):
+        releases = self._repo.get_releases()
+
+        if threshold:
+            return [
+                rel for rel in releases if semver.compare(threshold, rel.tag_name) == 1
+            ]
+
+        return releases.sort(reverse=True)[0:4]
+
+    def _manage_youtrack_card(self, commits, new_version, countries):
         release_state = self._config.youtrack["release_state"]
 
         issue_ids = self._youtrack.get_issue_ids(commits)
@@ -186,7 +270,7 @@ class Release:
                 try:
                     self._youtrack.comment(
                         issue_id,
-                        f"Deploy in production of {self._project} in country {country} done with the release {new_version}",
+                        f"Deploy in production of {self._project} in countries {countries} done with the release {new_version}",
                     )
                     if update_youtrack_state:
                         self._youtrack.update_state(issue_id, release_state)
@@ -228,29 +312,6 @@ class Release:
                     pass
         return repos_status
 
-    def _parse_available_countries(self):
-        pipelines = self._drone.parse_yaml()
-
-        if pipelines is None:
-            logger.error("The file .drone.yml was not found. Unable to continue.")
-            sys.exit(1)
-
-        countries = []
-        REGEX = re.compile(r"(?<=-).*?(?=-)")
-        for pipeline in pipelines:
-            if "name" in pipeline:
-                c = REGEX.findall(pipeline["name"])
-                if len(c) > 0 and c[0] is not None and c[0] not in countries:
-                    countries.append(c[0])
-
-        if len(countries) == 0:
-            logger.error(
-                "Can't determine which countries can be deployed. Try to run `suite-py deploy` instead."
-            )
-            sys.exit(1)
-
-        return countries
-
 
 def _bump_versions(current):
     return {
