@@ -5,7 +5,6 @@ import time
 import subprocess
 from subprocess import CalledProcessError
 import sys
-import re
 import requests
 import yaml
 
@@ -131,9 +130,8 @@ class DroneHandler:
         )
         sys.exit(-1)
 
-    def get_latest_production_deploy(self):
+    def get_latest_production_deploy(self, countries):
         with Halo(text="Contacting drone...", spinner="dots", color="magenta"):
-            REGEX = r"deploy-([a-z]+)-production"
             try:
                 builds = requests.get(
                     f"{BASEURL}/api/repos/primait/{self._repo}/builds?per_page=100",
@@ -141,11 +139,15 @@ class DroneHandler:
                 ).json()
                 builds = [
                     {
-                        "country": re.findall(REGEX, b["deploy_to"])[0],
-                        "version": b["params"]["DRONE_TAG"],
+                        "country": country,
+                        "version": max(
+                            b["params"]["DRONE_TAG"]
+                            for b in builds
+                            if b["event"] == "promote"
+                            and b["deploy_to"] == f"deploy-{country}-production"
+                        ),
                     }
-                    for b in builds
-                    if b["event"] == "promote" and re.search(REGEX, b["deploy_to"])
+                    for country in countries
                 ]
                 return builds
             except Exception as e:
